#include <curses.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <time.h>

#include "../include/error.h"

/*
 * Init of the printing system (ncurses)
 */
void init_print()
{
	initscr();
	cbreak();
	noecho();
	keypad(stdscr, TRUE);
	curs_set(0);
}

/*
 * Cleanup ncurses system
 */
void cleanup_print()
{
	endwin();
}

/*
 * Turn the st_mode field permissions into
 * a more readable thing. Like this :
 * drwxr-xr-x
 */
void get_permissions(mode_t st_mode, char *perm)
{
	int cpt;

	for (cpt = 0; cpt < 10; cpt++) {
		perm[cpt] = '-';
	}
	perm[cpt] = 0;

	if (S_ISDIR(st_mode)) {
		perm[0] = 'd';
	}
	else if (S_ISCHR(st_mode)) {
		perm[0] = 'c';
	}
	else if (S_ISBLK(st_mode)) {
		perm[0] = 'b';
	}
	else if (S_ISFIFO(st_mode)) {
		perm[0] = 'p';
	}
	else if (S_ISLNK(st_mode)) {
		perm[0] = 'l';
	}
	else if (S_ISSOCK(st_mode)) {
		perm[0] = 's';
	}

	if (st_mode & S_IRUSR) {
		perm[1] = 'r';
	}
	if (st_mode & S_IWUSR) {
		perm[2] = 'w';
	}
	if (st_mode & S_IXUSR) {
		perm[3] = 'x';
	}

	if (st_mode & S_IRGRP) {
		perm[4] = 'r';
	}
	if (st_mode & S_IWGRP) {
		perm[5] = 'w';
	}
	if (st_mode & S_IXGRP) {
		perm[6] = 'x';
	}

	if (st_mode & S_IROTH) {
		perm[7] = 'r';
	}
	if (st_mode & S_IWOTH) {
		perm[8] = 'w';
	}
	if (st_mode & S_IXOTH) {
		perm[9] = 'x';
	}
}

/*
 * Print informations about the file
 */
int print_file(const char *filename, int nline)
{
	struct stat stat_buf;
	struct passwd *pass_line;
	struct group *grp_line;
	char perms[11];

	if (stat(filename, &stat_buf) < 0) {
		return -ESTAT;
	}

	get_permissions(stat_buf.st_mode, perms);
	pass_line = getpwuid(stat_buf.st_uid);
	grp_line = getgrgid(stat_buf.st_gid);
	
	mvprintw(nline, 0, "%s", perms);

	if (!pass_line) {
		mvprintw(nline, 15, "%d", stat_buf.st_uid);
	}
	else {
		mvprintw(nline, 15, "%s", pass_line->pw_name);
	}

	if (!grp_line) {
		mvprintw(nline, 25, "%d", stat_buf.st_gid);
	}
	else {
		mvprintw(nline, 25, "%s", grp_line->gr_name);
	}

	mvprintw(nline, 35, "%d", stat_buf.st_size);
	mvprintw(nline, 50, "%s", ctime(&stat_buf.st_ctime));
	mvprintw(nline, 80, "%s", filename);

	return SUCCESS;
}

/*
 * Print informations on all the file
 * of the given directory
 */
int print_directory(const char *directory)
{
	DIR *currentDir;
	struct dirent *currentDirent;
	int cpt = 1;

	currentDir = opendir(directory);
	if (!currentDir) {
		return -EOPEN;
	}

	clear();

	attron(A_BOLD);
	mvprintw(0, 0, "Permissions");
	mvprintw(0, 15, "Uid");
	mvprintw(0, 25, "Gid");
	mvprintw(0, 35, "Size");
	mvprintw(0, 50, "Date");
	mvprintw(0, 80, "Name");
	attroff(A_BOLD);
	
	mvprintw(LINES-1, 0, "q to quit");

	
	while ((currentDirent = readdir(currentDir))) {
		if ((strcmp(currentDirent->d_name, ".") != 0) && (strcmp(currentDirent->d_name, "..") != 0)) {
			if (print_file(currentDirent->d_name, cpt) < 0) {
				mvprintw(LINES-2, 0, "There are some error in the result. Don't trust the output");
			}
			else {
				cpt++;
			}
		}
	}

	closedir(currentDir);

	refresh();

	return SUCCESS;
}
